@@ -496,6 +496,42 @@ impl<'g> Workspace<'g> {
         &self.inner.target_directory
     }
 
+    /// Returns the build directory in which intermediate build artifacts are
+    /// stored.
+    ///
+    /// This field is only available if the `Metadata` was generated by Cargo
+    /// 1.91 or later.
+    pub fn build_directory(&self) -> Option<&'g Utf8Path> {
+        self.inner.build_directory.as_deref()
+    }
+
+    /// Returns an iterator over the workspace default members.
+    ///
+    /// Default members are the packages that are built when `cargo build` is
+    /// run without any arguments in the workspace root.
+    ///
+    /// This field is only available if the `Metadata` was generated by Cargo
+    /// 1.71 or later. For older versions, this will return an empty iterator.
+    pub fn default_member_ids(&self) -> impl ExactSizeIterator<Item = &'g PackageId> + use<'g> {
+        self.inner.default_members.iter()
+    }
+
+    /// Returns an iterator over package metadatas for workspace default
+    /// members.
+    ///
+    /// Default members are the packages that are built when `cargo build` is
+    /// run without any arguments in the workspace root.
+    ///
+    /// This field is only available if the `Metadata` was generated by Cargo
+    /// 1.71 or later. For older versions, this will return an empty iterator.
+    pub fn default_members(&self) -> impl ExactSizeIterator<Item = PackageMetadata<'g>> + use<'g> {
+        let graph = self.graph;
+        self.inner
+            .default_members
+            .iter()
+            .map(move |id| graph.metadata(id).expect("valid package ID"))
+    }
+
     /// Returns the number of packages in this workspace.
     pub fn member_count(&self) -> usize {
         self.inner.members_by_path.len()
@@ -671,10 +707,12 @@ mod workspace_rayon {
 pub(super) struct WorkspaceImpl {
     pub(super) root: Utf8PathBuf,
     pub(super) target_directory: Utf8PathBuf,
+    pub(super) build_directory: Option<Utf8PathBuf>,
     pub(super) metadata_table: JsonValue,
     // This is a BTreeMap to allow presenting data in sorted order.
     pub(super) members_by_path: BTreeMap<Utf8PathBuf, PackageId>,
     pub(super) members_by_name: BTreeMap<Box<str>, PackageId>,
+    pub(super) default_members: Vec<PackageId>,
     // Cache for members by name (only used for proptests)
     #[cfg(feature = "proptest1")]
     pub(super) name_list: OnceCell<Vec<Box<str>>>,
@@ -1849,6 +1887,22 @@ impl<'g> PackageLink<'g> {
         &self.inner.version_req
     }
 
+    /// Returns the registry URL for this dependency, if specified.
+    ///
+    /// Returns `None` for dependencies from crates.io (the default registry) or
+    /// for dependencies without an explicit registry.
+    pub fn registry(&self) -> Option<&'g str> {
+        self.inner.registry.as_deref()
+    }
+
+    /// Returns the file system path for this dependency, if it is a path
+    /// dependency.
+    ///
+    /// Returns `None` for dependencies from registries or other sources.
+    pub fn path(&self) -> Option<&'g Utf8Path> {
+        self.inner.path.as_deref()
+    }
+
     /// Returns details about this dependency from the `[dependencies]` section.
     pub fn normal(&self) -> DependencyReq<'g> {
         DependencyReq {
@@ -1921,6 +1975,8 @@ pub(crate) struct PackageLinkImpl {
     pub(super) dep_name: String,
     pub(super) resolved_name: String,
     pub(super) version_req: VersionReq,
+    pub(super) registry: Option<String>,
+    pub(super) path: Option<Utf8PathBuf>,
     pub(super) normal: DependencyReqImpl,
     pub(super) build: DependencyReqImpl,
     pub(super) dev: DependencyReqImpl,
